# -*- coding: utf8 -*-
#-----------------------------------------------------------------------------
#  Copyright (c) 2010-2012 Brian Granger, Min Ragan-Kelley
#
#  This file is part of pyzmq
#
#  Distributed under the terms of the New BSD License.  The full license is in
#  the file COPYING.BSD, distributed as part of this software.
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

import sys
import time

from unittest import TestCase

import zmqpy
from zmqpy.eventloop import ioloop, zmqstream

class TestZMQStream(TestCase):

    def setUp(self):
        self.context = zmqpy.Context()
        self.socket = self.context.socket(zmqpy.REP)
        self.loop = ioloop.IOLoop.instance()
        self.stream = zmqstream.ZMQStream(self.socket)

    def tearDown(self):
        self.socket.close()
        self.context.term()

    def test_callable_check(self):
        """Ensure callable check works (py3k)."""

        self.stream.on_send(lambda *args: None)
        self.stream.on_recv(lambda *args: None)
        self.assertRaises(AssertionError, self.stream.on_recv, 1)
        self.assertRaises(AssertionError, self.stream.on_send, 1)
        self.assertRaises(AssertionError, self.stream.on_recv, zmqpy)




# coding: utf-8

from cffi import FFI

from .error import *

ffi = FFI()

errnos =  ['EADDRINUSE', 'EADDRNOTAVAIL', 'EAGAIN', 'ECONNREFUSED', 'EFAULT',
           'EFSM', 'EINPROGRESS', 'EINVAL', 'EMTHREAD', 'ENETDOWN', 'ENOBUFS',
           'ENOCOMPATPROTO', 'ENODEV', 'ENOMEM', 'ENOTSUP', 'EPROTONOSUPPORT',
           'ETERM', 'ENOTSOCK', 'EMSGSIZE', 'EAFNOSUPPORT', 'ENETUNREACH',
           'ECONNABORTED', 'ECONNRESET', 'ENOTCONN', 'ETIMEDOUT',
           'EHOSTUNREACH', 'ENETRESET']

zmq2_cons = ['ZMQ_MSG_MORE' , 'ZMQ_MSG_SHARED', 'ZMQ_MSG_MASK',
             'ZMQ_UPSTREAM', 'ZMQ_DOWNSTREAM', 'ZMQ_MCAST_LOOP',
             'ZMQ_RECOVERY_IVL_MSEC', 'ZMQ_NOBLOCK', 'ZMQ_HWM',
             'ZMQ_SWAP']

socket_cons = ['ZMQ_PAIR', 'ZMQ_PUB', 'ZMQ_SUB', 'ZMQ_REQ', 'ZMQ_REP',
               'ZMQ_DEALER', 'ZMQ_ROUTER', 'ZMQ_PULL', 'ZMQ_PUSH', 'ZMQ_XPUB',
               'ZMQ_XSUB', 'ZMQ_XREQ', 'ZMQ_XREP']

zmq_base_cons = ['ZMQ_VERSION', 'ZMQ_AFFINITY', 'ZMQ_IDENTITY', 'ZMQ_SUBSCRIBE',
                 'ZMQ_UNSUBSCRIBE', 'ZMQ_RATE', 'ZMQ_RECOVERY_IVL',
                 'ZMQ_SNDBUF', 'ZMQ_RCVBUF', 'ZMQ_RCVMORE', 'ZMQ_FD',
                 'ZMQ_EVENTS', 'ZMQ_TYPE', 'ZMQ_LINGER', 'ZMQ_RECONNECT_IVL',
                 'ZMQ_BACKLOG', 'ZMQ_RECONNECT_IVL_MAX', 'ZMQ_RCVTIMEO',
                 'ZMQ_SNDTIMEO', 'ZMQ_SNDMORE', 'ZMQ_POLLIN', 'ZMQ_POLLOUT',
                 'ZMQ_POLLERR', 'ZMQ_STREAMER', 'ZMQ_FORWARDER', 'ZMQ_QUEUE']

zmq3_cons = ['ZMQ_DONTWAIT', 'ZMQ_MORE', 'ZMQ_MAXMSGSIZE', 'ZMQ_SNDHWM',
             'ZMQ_RCVHWM', 'ZMQ_MULTICAST_HOPS', 'ZMQ_IPV4ONLY',
             'ZMQ_LAST_ENDPOINT', 'ZMQ_ROUTER_BEHAVIOR', 'ZMQ_TCP_KEEPALIVE',
             'ZMQ_TCP_KEEPALIVE_CNT', 'ZMQ_TCP_KEEPALIVE_IDLE',
             'ZMQ_TCP_KEEPALIVE_INTVL', 'ZMQ_TCP_ACCEPT_FILTER',
             'ZMQ_EVENT_CONNECTED', 'ZMQ_EVENT_CONNECT_DELAYED',
             'ZMQ_EVENT_CONNECT_RETRIED', 'ZMQ_EVENT_LISTENING',
             'ZMQ_EVENT_BIND_FAILED', 'ZMQ_EVENT_ACCEPTED',
             'ZMQ_EVENT_ACCEPT_FAILED', 'ZMQ_EVENT_CLOSED',
             'ZMQ_EVENT_CLOSE_FAILED']

core_functions = \
'''
void* zmq_init(int);
int zmq_term(void *context);

void* zmq_socket(void *context, int type);
int zmq_close(void *socket);

int zmq_bind(void *socket, const char *endpoint);
int zmq_connect(void *socket, const char *endpoint);

int zmq_errno(void);
'''

core22_functions = \
'''
int zmq_device(int device, void *frontend, void *backend);
'''

core32_functions = \
'''
int zmq_unbind(void *socket, const char *endpoint);
int zmq_disconnect(void *socket, const char *endpoint);
'''

message_functions = \
'''
typedef struct {
    void *content;
    unsigned char flags;
    unsigned char vsm_size;
    unsigned char vsm_data [30];
} zmq_msg_t;

typedef ... zmq_free_fn;

int zmq_msg_init(zmq_msg_t *msg);
int zmq_msg_init_size(zmq_msg_t *msg, size_t size);
int zmq_msg_init_data(zmq_msg_t *msg,
                      void *data,
                      size_t size,
                      zmq_free_fn *ffn,
                      void *hint);

size_t zmq_msg_size(zmq_msg_t *msg);
void *zmq_msg_data(zmq_msg_t *msg);
int zmq_msg_close(zmq_msg_t *msg);

int zmq_send(void *socket, zmq_msg_t *msg, int flags);
int zmq_recv(void *socket, zmq_msg_t *msg, int flags);
'''

message32_functions = \
'''
typedef struct {unsigned char _ [32];} zmq_msg_t;
typedef ... zmq_free_fn;

int zmq_msg_init(zmq_msg_t *msg);
int zmq_msg_init_size(zmq_msg_t *msg, size_t size);
int zmq_msg_init_data(zmq_msg_t *msg,
                      void *data,
                      size_t size,
                      zmq_free_fn *ffn,
                      void *hint);

size_t zmq_msg_size(zmq_msg_t *msg);
void *zmq_msg_data(zmq_msg_t *msg);
int zmq_msg_close(zmq_msg_t *msg);

int zmq_sendmsg(void *socket, zmq_msg_t *msg, int flags);
int zmq_recvmsg(void *socket, zmq_msg_t *msg, int flags);
'''

sockopt_functions = \
'''
int zmq_getsockopt(void *socket,
                   int option_name,
                   void *option_value,
                   size_t *option_len);

int zmq_setsockopt(void *socket,
                   int option_name,
                   const void *option_value,
                   size_t option_len);
'''

polling_functions = \
'''
typedef struct
{
    void *socket;
    int fd;
    short events;
    short revents;
} zmq_pollitem_t;

int zmq_poll(zmq_pollitem_t *items, int nitems, long timeout);
'''

extra_functions = \
'''
void* memcpy(void *restrict s1, const void *restrict s2, size_t n);
'''

def get_version():
    ffi_check = FFI()
    ffi_check.cdef('void zmq_version(int *major, int *minor, int *patch);')
    C_check_version = ffi_check.verify('#include <zmq.h>',
                                        libraries=['c', 'zmq'])
    major = ffi.new('int*')
    minor = ffi.new('int*')
    patch = ffi.new('int*')

    C_check_version.zmq_version(major, minor, patch)

    return (int(major[0]), int(minor[0]), int(patch[0]))


constant_names = []

zmq_version = get_version()[0]

if zmq_version == 2:
    constant_names = errnos + socket_cons + zmq_base_cons + zmq2_cons
else:
    constant_names = errnos + socket_cons + zmq_base_cons + zmq3_cons


def _make_defines(names):
    _names = []
    for name in names:
        define_line = "#define %s ..." % (name)
        _names.append(define_line)

    return "\n".join(_names)

constants = _make_defines(constant_names)

if zmq_version == 2:
    functions = ''.join([constants,
                         core_functions,
                         core22_functions,
                         message_functions,
                         sockopt_functions,
                         polling_functions,
                         extra_functions])
elif zmq_version == 3:
    functions = ''.join([constants,
                         core_functions,
                         core32_functions,
                         message32_functions,
                         sockopt_functions,
                         polling_functions,
                         extra_functions])
else:
    raise Exception("Bad ZMQ Install")

ffi.cdef(functions)

C = ffi.verify('''
    #include <stddef.h>
    #include <string.h>
    #include <zmq.h>
''', libraries=['c', 'zmq'])

nsp = new_sizet_pointer = lambda length: ffi.new('size_t*', length)

new_uint64_pointer = lambda: (ffi.new('uint64_t*'),
                              nsp(ffi.sizeof('uint64_t')))
new_int64_pointer = lambda: (ffi.new('int64_t*'),
                             nsp(ffi.sizeof('int64_t')))
new_int_pointer = lambda: (ffi.new('int*'),
                           nsp(ffi.sizeof('int')))
new_binary_data = lambda length: (ffi.new('char[%d]' % (length)),
                                  nsp(ffi.sizeof('char') * length))

value_uint64_pointer = lambda val : (ffi.new('uint64_t*', val),
                                     ffi.sizeof('uint64_t'))
value_int64_pointer = lambda val: (ffi.new('int64_t*', val),
                                   ffi.sizeof('int64_t'))
value_int_pointer = lambda val: (ffi.new('int*', val),
                                 ffi.sizeof('int'))
value_binary_data = lambda val, length: (ffi.new('char[%d]' % (length), val),
                                         ffi.sizeof('char') * length)



import zmqpy
from ctypes import get_errno

class ZMQError(Exception):
    def __init__(self, _errno=0):
        if not _errno:
            self._errno = get_errno()
        else:
            self._errno = _errno



# coding: utf-8

from ctypes import *
from ctypes.util import find_library

from .constants import *
from .error import *

czmq = CDLL(find_library("czmq"), use_errno=True)

if not czmq._name:
    raise ImportError('Cannot load czmq library')

class zmq_context(Structure):
    pass

class zmq_loop(Structure):
    pass

class zmq_pollitem_t(Structure):
    _fields_ = [("socket", c_void_p),
                ("f", c_int),
                ("events", c_short),
                ("revents", c_short)]

class zmq_msg_t(Structure):
    _fields_ = [('_',c_ubyte * 32)]

class zmq_frame(Structure):
    _fields_ = [('zmsg', zmq_msg_t),
                ('more', c_int)]

czmq.zctx_new.restype = POINTER(zmq_context)
czmq.zctx_new.argtypes = []

czmq.zctx_destroy.restype = None
czmq.zctx_destroy.argtypes = [c_void_p]

czmq.zctx_set_linger.restype = None
czmq.zctx_set_linger.argtypes = [c_void_p, c_int]

czmq.zctx_set_iothreads.restype = None
czmq.zctx_set_iothreads.argtypes = [c_void_p, c_int]

czmq.zsocket_new.restype = c_void_p
czmq.zsocket_new.argtypes = [c_void_p, c_int]

czmq.zsocket_destroy.restype = None
czmq.zsocket_destroy.argtypes = [c_void_p, c_void_p]

czmq.zsocket_connect.restype = None
czmq.zsocket_connect.argtypes = [c_void_p, c_char_p]

czmq.zsocket_bind.restype = None
czmq.zsocket_bind.argtypes = [c_void_p, c_char_p]

czmq.zstr_recv.restype = c_char_p
czmq.zstr_recv.argtypes = [c_void_p]

czmq.zstr_send.restype = c_int
czmq.zstr_send.argtypes = [c_void_p, c_char_p]

czmq.zstr_recv_nowait.restype = c_char_p
czmq.zstr_recv_nowait.argtypes = [c_void_p]

try:
    #trying zmq version 2
    czmq.zsocket_hwm.restype = c_int
    czmq.zsocket_hwm.argtypes = [c_void_p]

    czmq.zsocket_swap.restype = c_int
    czmq.zsocket_swap.argtypes = [c_void_p]

    czmq.zsocket_affinity.restype = c_int
    czmq.zsocket_affinity.argtypes = [c_void_p]

    czmq.zsocket_rate.restype = c_int
    czmq.zsocket_rate.argtypes = [c_void_p]

    czmq.zsocket_recovery_ivl.restype = c_int
    czmq.zsocket_recovery_ivl.argtypes = [c_void_p]

    czmq.zsocket_recovery_ivl_msec.restype = c_int
    czmq.zsocket_recovery_ivl_msec.argtypes = [c_void_p]

    czmq.zsocket_mcast_loop.restype = c_int
    czmq.zsocket_mcast_loop.argtypes = [c_void_p]

    czmq.zsocket_sndbuf.restype = c_int
    czmq.zsocket_sndbuf.argtypes = [c_void_p]

    czmq.zsocket_rcvbuf.restype = c_int
    czmq.zsocket_rcvbuf.argtypes = [c_void_p]

    czmq.zsocket_linger.restype = c_int
    czmq.zsocket_linger.argtypes = [c_void_p]

    czmq.zsocket_reconnect_ivl.restype = c_int
    czmq.zsocket_reconnect_ivl.argtypes = [c_void_p]

    czmq.zsocket_reconnect_ivl_max.restype = c_int
    czmq.zsocket_reconnect_ivl_max.argtypes = [c_void_p]

    czmq.zsocket_backlog.restype = c_int
    czmq.zsocket_backlog.argtypes = [c_void_p]

    czmq.zsocket_type.restype = c_int
    czmq.zsocket_type.argtypes = [c_void_p]

    czmq.zsocket_rcvmore.restype = c_int
    czmq.zsocket_rcvmore.argtypes = [c_void_p]

    czmq.zsocket_fd.restype = c_int
    czmq.zsocket_fd.argtypes = [c_void_p]

    czmq.zsocket_events.restype = c_int
    czmq.zsocket_events.argtypes = [c_void_p]

    czmq.zsocket_set_hwm.restype = None
    czmq.zsocket_set_hwm.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_swap.restype = None
    czmq.zsocket_set_swap.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_affinity.restype = None
    czmq.zsocket_set_affinity.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_identity.restype = None
    czmq.zsocket_set_identity.argtypes = [c_void_p, c_char_p]

    czmq.zsocket_set_rate.restype = None
    czmq.zsocket_set_rate.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_recovery_ivl.restype = None
    czmq.zsocket_set_recovery_ivl.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_recovery_ivl_msec.restype = None
    czmq.zsocket_set_recovery_ivl_msec.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_mcast_loop.restype = None
    czmq.zsocket_set_mcast_loop.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_sndbuf.restype = None
    czmq.zsocket_set_sndbuf.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_rcvbuf.restype = None
    czmq.zsocket_set_rcvbuf.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_linger.restype = None
    czmq.zsocket_set_linger.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_reconnect_ivl.restype = None
    czmq.zsocket_set_reconnect_ivl.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_reconnect_ivl_max.restype = None
    czmq.zsocket_set_reconnect_ivl_max.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_backlog.restype = None
    czmq.zsocket_set_backlog.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_subscribe.restype = None
    czmq.zsocket_set_subscribe.argtypes = [c_void_p, c_char_p]

    czmq.zsocket_set_unsubscribe.restype = None
    czmq.zsocket_set_unsubscribe.argtypes = [c_void_p, c_char_p]
except Exception as e:
    import logging

    log = logging.getLogger()
    log.error(e)

    #trying zmq version 3
    czmq.zsocket_sndhwm.restype = c_int
    czmq.zsocket_sndhwm.argtypes = [c_void_p]

    czmq.zsocket_rcvhwm.restype = c_int
    czmq.zsocket_rcvhwm.argtypes = [c_void_p]

    czmq.zsocket_affinity.restype = c_int
    czmq.zsocket_affinity.argtypes = [c_void_p]

    czmq.zsocket_rate.restype = c_int
    czmq.zsocket_rate.argtypes = [c_void_p]

    czmq.zsocket_recovery_ivl.restype = c_int
    czmq.zsocket_recovery_ivl.argtypes = [c_void_p]

    czmq.zsocket_sndbuf.restype = c_int
    czmq.zsocket_sndbuf.argtypes = [c_void_p]

    czmq.zsocket_rcvbuf.restype = c_int
    czmq.zsocket_rcvbuf.argtypes = [c_void_p]

    czmq.zsocket_linger.restype = c_int
    czmq.zsocket_linger.argtypes = [c_void_p]

    czmq.zsocket_reconnect_ivl.restype = c_int
    czmq.zsocket_reconnect_ivl.argtypes = [c_void_p]

    czmq.zsocket_reconnect_ivl_max.restype = c_int
    czmq.zsocket_reconnect_ivl_max.argtypes = [c_void_p]

    czmq.zsocket_backlog.restype = c_int
    czmq.zsocket_backlog.argtypes = [c_void_p]

    czmq.zsocket_maxmsgsize.restype = c_int
    czmq.zsocket_maxmsgsize.argtypes = [c_void_p]

    czmq.zsocket_type.restype = c_int
    czmq.zsocket_type.argtypes = [c_void_p]

    czmq.zsocket_rcvmore.restype = c_int
    czmq.zsocket_rcvmore.argtypes = [c_void_p]

    czmq.zsocket_fd.restype = c_int
    czmq.zsocket_fd.argtypes = [c_void_p]

    czmq.zsocket_events.restype = c_int
    czmq.zsocket_events.argtypes = [c_void_p]

    czmq.zsocket_set_sndhwm.restype = None
    czmq.zsocket_set_sndhwm.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_sndhwm.restype = None
    czmq.zsocket_set_sndhwm.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_rcvhwm.restype = None
    czmq.zsocket_set_rcvhwm.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_affinity.restype = None
    czmq.zsocket_set_affinity.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_identity.restype = None
    czmq.zsocket_set_identity.argtypes = [c_void_p, c_char_p]

    czmq.zsocket_set_rate.restype = None
    czmq.zsocket_set_rate.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_recovery_ivl.restype = None
    czmq.zsocket_set_recovery_ivl.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_sndbuf.restype = None
    czmq.zsocket_set_sndbuf.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_linger.restype = None
    czmq.zsocket_set_linger.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_reconnect_ivl.restype = None
    czmq.zsocket_set_reconnect_ivl.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_reconnect_ivl_max.restype = None
    czmq.zsocket_set_reconnect_ivl_max.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_backlog.restype = None
    czmq.zsocket_set_backlog.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_maxmsgsize.restype = None
    czmq.zsocket_set_maxmsgsize.argtypes = [c_void_p, c_int]

    czmq.zsocket_set_subscribe.restype = None
    czmq.zsocket_set_subscribe.argtypes = [c_void_p, c_char_p]

    czmq.zsocket_set_unsubscribe.restype = None
    czmq.zsocket_set_unsubscribe.argtypes = [c_void_p, c_char_p]

poller_callback_func = CFUNCTYPE(c_int,
                                 POINTER(zmq_loop),
                                 POINTER(zmq_pollitem_t),
                                 c_void_p)

czmq.zloop_new.restype = POINTER(zmq_loop)
czmq.zloop_new.argtypes = []

czmq.zloop_destroy.restype = None
czmq.zloop_destroy.argtypes = [POINTER(POINTER(zmq_loop))]

czmq.zloop_poller.restype = c_int
czmq.zloop_poller.argtypes = [
                                POINTER(zmq_loop),
                                POINTER(zmq_pollitem_t),
                                poller_callback_func,
                                c_void_p
                             ]

czmq.zloop_poller_end.restype = None
czmq.zloop_poller_end.argtypes = [POINTER(zmq_loop), POINTER(zmq_pollitem_t)]

czmq.zloop_start.restype = c_int
czmq.zloop_start.argtypes = [POINTER(zmq_loop)]

czmq.zloop_timer.restype = c_int
czmq.zloop_timer.argtypes = [
                                POINTER(zmq_loop),
                                c_size_t,
                                c_size_t,
                                poller_callback_func,
                                c_void_p
                            ]

czmq.zloop_set_verbose.restype = None
czmq.zloop_set_verbose.argtypes = [POINTER(zmq_loop), c_bool]

czmq.zframe_new.restype = POINTER(zmq_frame)
czmq.zframe_new.argtypes = [c_void_p, c_size_t]

czmq.zframe_recv.restype = POINTER(zmq_frame)
czmq.zframe_recv.argtypes = [c_void_p]

czmq.zframe_recv_nowait.restype = POINTER(zmq_frame)
czmq.zframe_recv_nowait.argtypes = [c_void_p]

czmq.zframe_send.restype = c_int
czmq.zframe_send.argtypes = [POINTER(POINTER(zmq_frame)), c_void_p, c_int]

czmq.zframe_data.restype = c_char_p
czmq.zframe_data.argtypes = [POINTER(zmq_frame)]



import unittest

class TestZmqpy(unittest.TestCase):
    def test_import_zmqpy(self):
        try:
            import zmqpy
            from zmqpy import Context, Socket
        except ImportError as ie:
            self.fail(ie.message)

class TestContext(unittest.TestCase):
    def tearDown(self):
        from zmqpy import Context
        c = Context()
        c.term()

    def test_context_init(self):
        from zmqpy import Context

        c = Context()
        cc = Context()

        assert type(c) == Context
        assert c.zmq_ctx
        assert c.n_sockets == 0
        assert c._sockets == {}
        assert c.closed == False
        assert c.iothreads == 1

        assert id(c.__dict__) == id(cc.__dict__)

    def test_context_term(self):
        from zmqpy import Context

        c = Context()

        c.term()

        assert c.closed
        assert c.zmq_ctx == None

    def test_context_socket(self):
        from zmqpy import Context, PAIR

        c = Context()
        socket = c.socket(PAIR)

        assert socket
        assert c.n_sockets == 1
        assert len(c._sockets) == 1
        assert not c.closed

    def test_context_socket_term(self):
        from zmqpy import Context, PAIR
        c = Context()
        socket = c.socket(PAIR)

        assert socket
        assert c.n_sockets == 1
        assert len(c._sockets) == 1
        assert not c.closed

        c.term()

        assert c.n_sockets == 0
        assert len(c._sockets) == 0
        assert socket.closed

class TestSocket(unittest.TestCase):
    def tearDown(self):
        from zmqpy import Context
        c = Context()
        c.term()

    def test_socket_bind(self):
        from zmqpy import Context, PAIR
        c = Context()
        socket = c.socket(PAIR)

        bind = socket.bind('tcp://*:3333')
        assert bind == 0

        socket.close()

    def test_socket_connect(self):
        from zmqpy import Context, PAIR
        c = Context()
        sender = c.socket(PAIR)
        receiver = c.socket(PAIR)

        bind = receiver.bind('tcp://*:3333')
        assert bind == 0

        connect = sender.connect('tcp://127.0.0.1:3333')
        assert connect == 0

        sender.close()
        receiver.close()

    def test_socket_disconnected_send(self):
        from zmqpy import Context, PAIR, NOBLOCK
        c = Context()
        socket = c.socket(PAIR)

        ret = socket.send("zmqpy test message", NOBLOCK)

        assert ret == -1
        assert socket.last_errno > 0

        socket.close()

    def test_socket_connected_send(self):
        from zmqpy import Context, PAIR
        from zmqpy._cffi import zmq_version
        c = Context()
        sender = c.socket(PAIR)
        receiver = c.socket(PAIR)

        bind = receiver.bind('tcp://*:3333')
        connect = sender.connect('tcp://127.0.0.1:333')

        message = "zmqpy test message"
        ret = sender.send(message)

        if zmq_version == 2:
            assert ret == 0
        else:
            assert ret == len(message)

        assert sender.last_errno == None

        sender.close()
        receiver.close()

    def test_socket_connected_recv(self):
        from zmqpy import Context, PAIR
        from zmqpy._cffi import zmq_version

        c = Context()
        sender = c.socket(PAIR)
        receiver = c.socket(PAIR)

        bind = receiver.bind('tcp://*:3333')
        connect = sender.connect('tcp://127.0.0.1:3333')

        assert bind == 0
        assert connect == 0

        ret = sender.send("zmqpy test message")
        if zmq_version == 2:
            assert ret == 0
        else:
            assert ret == 18

        assert sender.last_errno == None

        import time
        time.sleep(0.2)
        message = receiver.recv()

        assert sender.last_errno == None
        assert message == "zmqpy test message"

        sender.close()
        receiver.close()

    def test_socket_setsockopt_uint64(self):
        from zmqpy._cffi import zmq_version
        if zmq_version == 2:
            from zmqpy import Context, PAIR, HWM
        else:
            from zmqpy import Context, PAIR, RCVHWM
            HWM = RCVHWM

        c = Context()
        sender = c.socket(PAIR)

        rc = sender.setsockopt(HWM, 10)
        assert rc == 0
        sender.close()

    def test_socket_getsockopt_uint64(self):
        from zmqpy._cffi import zmq_version
        if zmq_version == 2:
            from zmqpy import Context, PAIR, HWM
        else:
            from zmqpy import Context, PAIR, RCVHWM
            HWM = RCVHWM

        c = Context()
        sender = c.socket(PAIR)

        rc = sender.setsockopt(HWM, 10)
        assert rc == 0

        opt_value = sender.getsockopt(HWM)
        assert opt_value != -1
        assert opt_value == 10
        sender.close()

    def test_socket_setsockopt_binary(self):
        from zmqpy import Context, PAIR, IDENTITY
        c = Context()
        sender = c.socket(PAIR)

        rc = sender.setsockopt(IDENTITY, "id")
        assert rc == 0
        sender.close()

    def test_socket_getsockopt_binary(self):
        from zmqpy import Context, PAIR, IDENTITY
        c = Context()
        sender = c.socket(PAIR)

        rc = sender.setsockopt(IDENTITY, "id")
        assert rc == 0

        opt_value = sender.getsockopt(IDENTITY, length=2)
        assert opt_value != -1
        assert opt_value == "id"
        sender.close()

    def test_socket_setsockopt_int64(self):
        from zmqpy import Context, PAIR, RATE
        c = Context()
        sender = c.socket(PAIR)

        rc = sender.setsockopt(RATE, 200)
        assert rc == 0
        sender.close()

    def test_socket_getsockopt_int64(self):
        from zmqpy import Context, PAIR, RATE
        c = Context()
        sender = c.socket(PAIR)

        rc = sender.setsockopt(RATE, 200)
        assert rc == 0

        opt_value = sender.getsockopt(RATE)
        assert opt_value != -1
        assert opt_value == 200
        sender.close()

    def test_socket_setsockopt_int(self):
        from zmqpy import Context, PAIR, LINGER
        c = Context()
        sender = c.socket(PAIR)

        rc = sender.setsockopt(LINGER, 10)
        assert rc == 0
        sender.close()

    def test_socket_getsockopt_int(self):
        from zmqpy import Context, PAIR, LINGER
        c = Context()
        sender = c.socket(PAIR)

        rc = sender.setsockopt(LINGER, 10)
        assert rc == 0

        opt_value = sender.getsockopt(LINGER)
        assert opt_value != -1
        assert opt_value == 10
        sender.close()



#
# Copyright 2010 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

'''StackContext allows applications to maintain threadlocal-like state
that follows execution as it moves to other execution contexts.

The motivating examples are to eliminate the need for explicit
async_callback wrappers (as in tornado.web.RequestHandler), and to
allow some additional context to be kept for logging.

This is slightly magic, but it's an extension of the idea that an exception
handler is a kind of stack-local state and when that stack is suspended
and resumed in a new context that state needs to be preserved.  StackContext
shifts the burden of restoring that state from each call site (e.g.
wrapping each AsyncHTTPClient callback in async_callback) to the mechanisms
that transfer control from one context to another (e.g. AsyncHTTPClient
itself, IOLoop, thread pools, etc).

Example usage::

    @contextlib.contextmanager
    def die_on_error():
        try:
            yield
        except Exception:
            logging.error("exception in asynchronous operation",exc_info=True)
            sys.exit(1)

    with StackContext(die_on_error):
        # Any exception thrown here *or in callback and its desendents*
        # will cause the process to exit instead of spinning endlessly
        # in the ioloop.
        http_client.fetch(url, callback)
    ioloop.start()

Most applications shouln't have to work with `StackContext` directly.
Here are a few rules of thumb for when it's necessary:

* If you're writing an asynchronous library that doesn't rely on a
  stack_context-aware library like `tornado.ioloop` or `tornado.iostream`
  (for example, if you're writing a thread pool), use
  `stack_context.wrap()` before any asynchronous operations to capture the
  stack context from where the operation was started.

* If you're writing an asynchronous library that has some shared
  resources (such as a connection pool), create those shared resources
  within a ``with stack_context.NullContext():`` block.  This will prevent
  ``StackContexts`` from leaking from one request to another.

* If you want to write something like an exception handler that will
  persist across asynchronous calls, create a new `StackContext` (or
  `ExceptionStackContext`), and make your asynchronous calls in a ``with``
  block that references your `StackContext`.
'''

from __future__ import absolute_import, division, with_statement

import contextlib
import functools
import sys
import threading

try:
    from itertools import izip
except ImportError:
    izip = zip   # In Python 3, zip produces an iterator

class _State(threading.local):
    def __init__(self):
        self.contexts = ()
_state = _State()


class StackContext(object):
    '''Establishes the given context as a StackContext that will be transferred.

    Note that the parameter is a callable that returns a context
    manager, not the context itself.  That is, where for a
    non-transferable context manager you would say::

      with my_context():

    StackContext takes the function itself rather than its result::

      with StackContext(my_context):
    '''
    def __init__(self, context_factory):
        self.context_factory = context_factory

    # Note that some of this code is duplicated in ExceptionStackContext
    # below.  ExceptionStackContext is more common and doesn't need
    # the full generality of this class.
    def __enter__(self):
        self.old_contexts = _state.contexts
        # _state.contexts is a tuple of (class, arg) pairs
        _state.contexts = (self.old_contexts +
                           ((StackContext, self.context_factory),))
        try:
            self.context = self.context_factory()
            self.context.__enter__()
        except Exception:
            _state.contexts = self.old_contexts
            raise

    def __exit__(self, type, value, traceback):
        try:
            return self.context.__exit__(type, value, traceback)
        finally:
            _state.contexts = self.old_contexts


class ExceptionStackContext(object):
    '''Specialization of StackContext for exception handling.

    The supplied exception_handler function will be called in the
    event of an uncaught exception in this context.  The semantics are
    similar to a try/finally clause, and intended use cases are to log
    an error, close a socket, or similar cleanup actions.  The
    exc_info triple (type, value, traceback) will be passed to the
    exception_handler function.

    If the exception handler returns true, the exception will be
    consumed and will not be propagated to other exception handlers.
    '''
    def __init__(self, exception_handler):
        self.exception_handler = exception_handler

    def __enter__(self):
        self.old_contexts = _state.contexts
        _state.contexts = (self.old_contexts +
                           ((ExceptionStackContext, self.exception_handler),))

    def __exit__(self, type, value, traceback):
        try:
            if type is not None:
                return self.exception_handler(type, value, traceback)
        finally:
            _state.contexts = self.old_contexts


class NullContext(object):
    '''Resets the StackContext.

    Useful when creating a shared resource on demand (e.g. an AsyncHTTPClient)
    where the stack that caused the creating is not relevant to future
    operations.
    '''
    def __enter__(self):
        self.old_contexts = _state.contexts
        _state.contexts = ()

    def __exit__(self, type, value, traceback):
        _state.contexts = self.old_contexts


class _StackContextWrapper(functools.partial):
    pass


def wrap(fn):
    '''Returns a callable object that will restore the current StackContext
    when executed.

    Use this whenever saving a callback to be executed later in a
    different execution context (either in a different thread or
    asynchronously in the same thread).
    '''
    if fn is None or fn.__class__ is _StackContextWrapper:
        return fn
    # functools.wraps doesn't appear to work on functools.partial objects
    #@functools.wraps(fn)

    def wrapped(callback, contexts, *args, **kwargs):
        if contexts is _state.contexts or not contexts:
            callback(*args, **kwargs)
            return
        if not _state.contexts:
            new_contexts = [cls(arg) for (cls, arg) in contexts]
        # If we're moving down the stack, _state.contexts is a prefix
        # of contexts.  For each element of contexts not in that prefix,
        # create a new StackContext object.
        # If we're moving up the stack (or to an entirely different stack),
        # _state.contexts will have elements not in contexts.  Use
        # NullContext to clear the state and then recreate from contexts.
        elif (len(_state.contexts) > len(contexts) or
            any(a[1] is not b[1]
                for a, b in izip(_state.contexts, contexts))):
            # contexts have been removed or changed, so start over
            new_contexts = ([NullContext()] +
                            [cls(arg) for (cls,arg) in contexts])
        else:
            new_contexts = [cls(arg)
                            for (cls, arg) in contexts[len(_state.contexts):]]
        if len(new_contexts) > 1:
            with _nested(*new_contexts):
                callback(*args, **kwargs)
        elif new_contexts:
            with new_contexts[0]:
                callback(*args, **kwargs)
        else:
            callback(*args, **kwargs)
    if _state.contexts:
        return _StackContextWrapper(wrapped, fn, _state.contexts)
    else:
        return _StackContextWrapper(fn)


@contextlib.contextmanager
def _nested(*managers):
    """Support multiple context managers in a single with-statement.

    Copied from the python 2.6 standard library.  It's no longer present
    in python 3 because the with statement natively supports multiple
    context managers, but that doesn't help if the list of context
    managers is not known until runtime.
    """
    exits = []
    vars = []
    exc = (None, None, None)
    try:
        for mgr in managers:
            exit = mgr.__exit__
            enter = mgr.__enter__
            vars.append(enter())
            exits.append(exit)
        yield vars
    except:
        exc = sys.exc_info()
    finally:
        while exits:
            exit = exits.pop()
            try:
                if exit(*exc):
                    exc = (None, None, None)
            except:
                exc = sys.exc_info()
        if exc != (None, None, None):
            # Don't rely on sys.exc_info() still containing
            # the right information. Another exception may
            # have been raised and caught by an exit method
            raise exc




# coding: utf-8

import random

from ._cffi import C, ffi, zmq_version, new_uint64_pointer, \
                                        new_int64_pointer, \
                                        new_int_pointer, \
                                        new_binary_data, \
                                        value_uint64_pointer, \
                                        value_int64_pointer, \
                                        value_int_pointer, \
                                        value_binary_data

from .constants import *
from .error import *
from .utils import jsonapi
from .utils.strtypes import bytes, unicode

class Context(object):
    _state = {}
    def __init__(self, iothreads=1):
        if not iothreads > 0:
            raise ZMQError(EINVAL)

        self.__dict__ = self._state

        self.zmq_ctx = C.zmq_init(iothreads)
        self.iothreads = iothreads
        self._closed = False
        self.n_sockets = 0
        self.max_sockets = 32
        self._sockets = {}
        self.sockopts = {LINGER: 1}
        self.linger = 1

    def term(self):
        if self.closed:
            return

        for k, s in self._sockets.items():
            if not s.closed:
                s.close()
            del self._sockets[k]

        C.zmq_term(self.zmq_ctx)
        self.zmq_ctx = None
        self._closed = True
        self.n_sockets = 0

    @property
    def closed(self):
        return self._closed

    def _add_socket(self, socket):
        self._sockets[self.n_sockets] = socket
        self.n_sockets += 1

        return self.n_sockets

    def _rm_socket(self, n):
        del self._sockets[n]

    def socket(self, sock_type):
        if self._closed:
            raise ZMQError(ENOTSUP)

        socket = Socket(self, sock_type)
        for option, option_value in self.sockopts.items():
            socket.setsockopt(option, option_value)

        return socket

    def set_linger(self, value):
        self.sockopts[LINGER] = value
        self.linger = value

def new_pointer_from_opt(option, length=0):
    if option in uint64_opts:
        return new_uint64_pointer()
    elif option in int64_opts:
        return new_int64_pointer()
    elif option in int_opts:
        return new_int_pointer()
    elif option in binary_opts:
        return new_binary_data(length)
    else:
        raise ValueError('Invalid option')

def value_from_opt_pointer(option, opt_pointer, length=0):
    if option in uint64_opts:
        return int(opt_pointer[0])
    elif option in int64_opts:
        return int(opt_pointer[0])
    elif option in int_opts:
        return int(opt_pointer[0])
    elif option in binary_opts:
        return ffi.string(opt_pointer)
    else:
        raise ValueError('Invalid option')

def initialize_opt_pointer(option, value, length=0):
    if option in uint64_opts:
        return value_uint64_pointer(value)
    elif option in int64_opts:
        return value_int64_pointer(value)
    elif option in int_opts:
        return value_int_pointer(value)
    elif option in binary_opts:
        return value_binary_data(value, length)
    else:
        raise ValueError('Invalid option')

class Socket(object):
    def __init__(self, context, sock_type):
        self.context = context
        self.sock_type = sock_type
        self.zmq_socket = C.zmq_socket(context.zmq_ctx, sock_type)
        if not self.zmq_socket:
            raise ZMQError()
        self._closed = False
        self._attrs = {}
        self.n = self.context._add_socket(self)
        self.last_errno = None

    @property
    def closed(self):
        return self._closed

    def close(self):
        if not self._closed:
            C.zmq_close(self.zmq_socket)
            self._closed = True

    def bind(self, address):
        ret = C.zmq_bind(self.zmq_socket, address)
        return ret

    def connect(self, address):
        ret = C.zmq_connect(self.zmq_socket, address)
        return ret

    def setsockopt(self, option, value):
        length = None
        if isinstance(value, str):
            length = len(value)
        low_level_data = initialize_opt_pointer(option, value, length)
        low_level_value_pointer = low_level_data[0]
        low_level_sizet = low_level_data[1]
        ret = C.zmq_setsockopt(self.zmq_socket,
                                option,
                                ffi.cast('void*', low_level_value_pointer),
                                low_level_sizet)
        return ret

    def setsockopt_string(self, option, value, encoding='utf-8'):
        if not isinstance(value, unicode):
            raise TypeError("unicode strings only")
        return self.setsockopt(option, value.encode(encoding))

    setsockopt_unicode = setsockopt_string

    def getsockopt(self, option, length=0):
        low_level_data = new_pointer_from_opt(option, length=length)
        low_level_value_pointer = low_level_data[0]
        low_level_sizet_pointer = low_level_data[1]

        ret = C.zmq_getsockopt(self.zmq_socket,
                               option,
                               low_level_value_pointer,
                               low_level_sizet_pointer)

        if ret < 0:
            self.last_errno = C.zmq_errno()
            return -1

        return value_from_opt_pointer(option, low_level_value_pointer)

    def send(self, message, flags=0, copy=False, track=False):
        zmq_msg = ffi.new('zmq_msg_t*')

        c_message = ffi.new('char[]', message)
        C.zmq_msg_init_size(zmq_msg, len(message))
        C.memcpy(C.zmq_msg_data(zmq_msg), c_message, len(message))

        if zmq_version == 2:
            ret = C.zmq_send(self.zmq_socket, zmq_msg, flags)
        else:
            ret = C.zmq_sendmsg(self. zmq_socket, zmq_msg, flags)

        C.zmq_msg_close(zmq_msg)
        if ret < 0:
            self.last_errno = C.zmq_errno()

        return ret

    def send_json(self, obj, flags=0, copy=False, track=False):
        if jsonapi.jsonmod is None:
            raise ImportError('jsonlib{1,2}, json or simplejson library is required.')
        else:
            msg = jsonapi.dumps(obj)
            return self.send(msg, flags, copy, track)

    def recv(self, flags=0, copy=False, track=False):
        zmq_msg = ffi.new('zmq_msg_t*')
        C.zmq_msg_init(zmq_msg)

        if zmq_version == 2:
            ret = C.zmq_recv(self.zmq_socket, zmq_msg, flags)
        else:
            ret = C.zmq_recvmsg(self.zmq_socket, zmq_msg, flags)

        if ret < 0:
            C.zmq_msg_close(zmq_msg)
            raise zmqpy.ZMQError(_errno=C.zmq_errno())

        value = ffi.buffer(C.zmq_msg_data(zmq_msg), int(C.zmq_msg_size(zmq_msg)))[:]

        C.zmq_msg_close(zmq_msg)

        return value

    def recv_json(self, flags=0, copy=False, track=False):
        if jsonapi.jsonmod is None:
            raise ImportError('jsonlib{1,2}, json or simplejson library is required.')
        else:
            msg = self.recv(flags, copy, track)
            return jsonapi.loads(msg)

    # Following methods from pyzmq.pysocket

    def bind_to_random_port(self, addr, min_port=49152, max_port=65536, max_tries=100):
        """s.bind_to_random_port(addr, min_port=49152, max_port=65536, max_tries=100)

        Bind this socket to a random port in a range.

        Parameters
        ----------
        addr : str
            The address string without the port to pass to ``Socket.bind()``.
        min_port : int, optional
            The minimum port in the range of ports to try (inclusive).
        max_port : int, optional
            The maximum port in the range of ports to try (exclusive).
        max_tries : int, optional
            The maximum number of bind attempts to make.

        Returns
        -------
        port : int
            The port the socket was bound to.

        Raises
        ------
        ZMQBindError
            if `max_tries` reached before successful bind
        """
        for i in range(max_tries):
            try:
                port = random.randrange(min_port, max_port)
                self.bind('%s:%s' % (addr, port))
            except ZMQError as exception:
                if not exception.errno == zmq.EADDRINUSE:
                    raise
            else:
                return port
        raise ZMQBindError("Could not bind socket to random port.")

    def send_multipart(self, msg_parts, flags=0, copy=True, track=False):
        """s.send_multipart(msg_parts, flags=0, copy=True, track=False)

        Send a sequence of buffers as a multipart message.

        Parameters
        ----------
        msg_parts : iterable
            A sequence of objects to send as a multipart message. Each element
            can be any sendable object (Frame, bytes, buffer-providers)
        flags : int, optional
            SNDMORE is handled automatically for frames before the last.
        copy : bool, optional
            Should the frame(s) be sent in a copying or non-copying manner.
        track : bool, optional
            Should the frame(s) be tracked for notification that ZMQ has
            finished with it (ignored if copy=True).

        Returns
        -------
        None : if copy or not track
        MessageTracker : if track and not copy
            a MessageTracker object, whose `pending` property will
            be True until the last send is completed.
        """
        for msg in msg_parts[:-1]:
            self.send(msg, SNDMORE|flags, copy=copy, track=track)
        # Send the last part without the extra SNDMORE flag.
        return self.send(msg_parts[-1], flags, copy=copy, track=track)

    def recv_multipart(self, flags=0, copy=True, track=False):
        """s.recv_multipart(flags=0, copy=True, track=False)

        Receive a multipart message as a list of bytes or Frame objects.

        Parameters
        ----------
        flags : int, optional
            Any supported flag: NOBLOCK. If NOBLOCK is set, this method
            will raise a ZMQError with EAGAIN if a message is not ready.
            If NOBLOCK is not set, then this method will block until a
            message arrives.
        copy : bool, optional
            Should the message frame(s) be received in a copying or non-copying manner?
            If False a Frame object is returned for each part, if True a copy of
            the bytes is made for each frame.
        track : bool, optional
            Should the message frame(s) be tracked for notification that ZMQ has
            finished with it? (ignored if copy=True)
        Returns
        -------
        msg_parts : list
            A list of frames in the multipart message; either Frames or bytes,
            depending on `copy`.

        """
        parts = [self.recv(flags, copy=copy, track=track)]
        # have first part already, only loop while more to receive
        while self.getsockopt(RCVMORE):
            part = self.recv(flags, copy=copy, track=track)
            parts.append(part)

        return parts

    def send_string(self, u, flags=0, copy=False, track=False, encoding='utf-8'):
        """send a Python unicode string as a message with an encoding
        0MQ communicates with raw bytes, so you must encode/decode
        text (unicode on py2, str on py3) around 0MQ.

        Parameters
        ----------
        u : Python unicode string (unicode on py2, str on py3)
        The unicode string to send.
        flags : int, optional
        Any valid send flag.
        encoding : str [default: 'utf-8']
        The encoding to be used
        """
        if not isinstance(u, basestring):
            raise TypeError("unicode/str objects only")
        return self.send(u.encode(encoding), flags=flags, copy=copy, track=track)
    
    send_unicode = send_string

    def recv_string(self, flags=0, track=False, encoding='utf-8'):
        """receive a unicode string, as sent by send_string
        Parameters
        ----------
        flags : int
        Any valid recv flag.
        encoding : str [default: 'utf-8']
        The encoding to be used

        Returns
        -------
        s : unicode string (unicode on py2, str on py3)
        The Python unicode string that arrives as encoded bytes.
        """
        msg = self.recv(flags=flags, copy=False, track=track)
        return codecs.decode(msg.bytes, encoding)
    
    recv_unicode = recv_string

def _make_zmq_pollitem(socket, flags):
    zmq_socket = socket.zmq_socket
    zmq_pollitem = ffi.new('zmq_pollitem_t*')
    zmq_pollitem.socket = zmq_socket
    zmq_pollitem.fd = 0
    zmq_pollitem.events = flags
    zmq_pollitem.revents = 0
    return zmq_pollitem[0]

def _make_zmq_pollitem_fromfd(socket_fd, flags):
    zmq_pollitem = ffi.new('zmq_pollitem_t*')
    zmq_pollitem.socket = ffi.NULL
    zmq_pollitem.fd = socket_fd
    zmq_pollitem.events = flags
    zmq_pollitem.revents = 0
    return zmq_pollitem[0]

def _cffi_poll(zmq_pollitem_list, poller, timeout=-1):
    if zmq_version == 2:
        timeout = timeout * 1000
    items = ffi.new('zmq_pollitem_t[]', zmq_pollitem_list)
    list_length = ffi.cast('int', len(zmq_pollitem_list))
    c_timeout = ffi.cast('long', timeout)
    C.zmq_poll(items, list_length, c_timeout)
    result = []
    for index in range(len(items)):
        if items[index].revents > 0:
            if not items[index].socket == ffi.NULL:
                result.append((poller._sockets[items[index].socket],
                            items[index].revents))
            else:
                result.append((items[index].fd, items[index].revents))
    return result

def _poll(sockets, timeout):
    cffi_pollitem_list = []
    low_level_to_socket_obj = {}
    for item in sockets:
        low_level_to_socket_obj[item[0].zmq_socket] = item
        cffi_pollitem_list.append(_make_zmq_pollitem(item[0], item[1]))
    items = ffi.new('zmq_pollitem_t[]', cffi_pollitem_list)
    list_length = ffi.cast('int', len(cffi_pollitem_list))
    c_timeout = ffi.cast('long', timeout)
    C.zmq_poll(items, list_length, c_timeout)
    result = []
    for index in range(len(items)):
        if items[index].revents > 0:
            result.append((low_level_to_socket_obj[items[index].socket][0],
                           items[index].revents))
    return result

class Poller(object):
    def __init__(self):
        self.sockets_flags = {}
        self._sockets = {}
        self.c_sockets = {}

    @property
    def sockets(self):
        return self.sockets_flags

    def register(self, socket, flags=POLLIN|POLLOUT):
        if flags:
            self.sockets_flags[socket] = flags
            if isinstance(socket, int):
                self.c_sockets[socket] = _make_zmq_pollitem_fromfd(socket, flags)
            else:
                self.c_sockets[socket] =  _make_zmq_pollitem(socket, flags)
                self._sockets[socket.zmq_socket] = socket
        elif socket in self.sockets_flags:
            # uregister sockets registered with no events
            self.unregister(socket)
        else:
            # ignore new sockets with no events
            pass

    def modify(self, socket, flags=POLLIN|POLLOUT):
        self.register(socket, flags)

    def unregister(self, socket):
        del self.sockets_flags[socket]
        del self.c_sockets[socket]

        if not isinstance(socket, int):
            del self._sockets[socket.zmq_socket]

    def poll(self, timeout=None):
        if timeout is None:
            timeout = -1

        timeout = int(timeout)
        if timeout < 0:
            timeout = -1

        items =  _cffi_poll(self.c_sockets.values(),
                            self,
                            timeout=timeout)

        return items

def select(rlist, wlist, xlist, timeout=None):
    if timeout is None:
        timeout = -1
    # Convert from sec -> us for zmq_poll.
    # zmq_poll accepts 3.x style timeout in ms
    timeout = int(timeout*1000.0)
    if timeout < 0:
        timeout = -1
    sockets = []
    for s in set(rlist + wlist + xlist):
        flags = 0
        if s in rlist:
            flags |= POLLIN
        if s in wlist:
            flags |= POLLOUT
        if s in xlist:
            flags |= POLLERR
        sockets.append((s, flags))
    return_sockets = _poll(sockets, timeout)
    rlist, wlist, xlist = [], [], []
    for s, flags in return_sockets:
        if flags & POLLIN:
            rlist.append(s)
        if flags & POLLOUT:
            wlist.append(s)
        if flags & POLLERR:
            xlist.append(s)
    return rlist, wlist, xlist



#
#    Copyright (c) 2010 Brian E. Granger
#
#    This file is part of pyzmq.
#
#    pyzmq is free software; you can redistribute it and/or modify it under
#    the terms of the Lesser GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    pyzmq is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    Lesser GNU General Public License for more details.
#
#    You should have received a copy of the Lesser GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------
import sys

from unittest import TestCase
from threading import Thread

import zmqpy as zmq
from zmqpy.utils import jsonapi

class BaseZMQTestCase(TestCase):
    def setUp(self):
        self.context = zmq.Context()
        self.sockets = []

    def tearDown(self):
        contexts = set([self.context])
        while self.sockets:
            sock = self.sockets.pop()
            contexts.add(sock.context)  # in case additional contexts
            sock.close()                # are created
        for ctx in contexts:
            t = Thread(target=ctx.term)
            t.start()
            t.join(timeout=2)
            if sys.version[:3] == '2.5':
                t.is_alive = t.isAlive
            if t.is_alive():
                raise RuntimeError("context could not terminate, \
                                open sockets likely remain in test")

    def create_bound_pair(self, type1, type2,
                          interface='tcp://127.0.0.1'):
        """Create a bound socket pair using a random port."""
        s1 = zmq.Socket(self.context, type1)
        s1.setsockopt(zmq.LINGER, 5)

        s2 = zmq.Socket(self.context, type2)
        s2.setsockopt(zmq.LINGER, 5)

        port=3333
        s1.bind('%s:%s' % (interface, port))
        s2.connect('%s:%s' % (interface, port))

        self.sockets.extend([s1,s2])
        return s1, s2

    def assertRaisesErrno(self, errno, func, *args, **kwargs):
        try:
            func(*args, **kwargs)
        except zmq.ZMQError as e:
            self.assertEqual(e._errno, errno, "wrong error raised, \ expected '%s' \
got '%s'" % (zmq.ZMQError(errno), zmq.ZMQError(e._errno)))
        else:
            self.fail("Function did not raise any error")


    def ping_pong(self, s1, s2, msg):
        s1.send(msg)
        msg2 = s2.recv()
        s2.send(msg2)
        msg3 = s1.recv()
        return msg3

    def ping_pong_json(self, s1, s2, o):
        if jsonapi.jsonmod is None:
            raise SkipTest("No json library")
        s1.send_json(o)
        o2 = s2.recv_json()
        s2.send_json(o2)
        o3 = s1.recv_json()
        return o3

    def ping_pong_pyobj(self, s1, s2, o):
        s1.send_pyobj(o)
        o2 = s2.recv_pyobj()
        s2.send_pyobj(o2)
        o3 = s1.recv_pyobj()
        return o3



#
#    Copyright (c) 2010 Brian E. Granger
#
#    This file is part of pyzmq.
#
#    pyzmq is free software; you can redistribute it and/or modify it under
#    the terms of the Lesser GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    pyzmq is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    Lesser GNU General Public License for more details.
#
#    You should have received a copy of the Lesser GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

import unittest
import zmqpy
import time

from zmqpy.utils.strtypes import asbytes
from .__init__ import BaseZMQTestCase

#-----------------------------------------------------------------------------
# Tests
#-----------------------------------------------------------------------------


class TestContext(BaseZMQTestCase):

    def test_init(self):
        c1 = zmqpy.Context()
        self.assert_(isinstance(c1, zmqpy.Context))
        del c1
        c2 = zmqpy.Context()
        self.assert_(isinstance(c2, zmqpy.Context))
        del c2
        c3 = zmqpy.Context()
        self.assert_(isinstance(c3, zmqpy.Context))
        del c3

    def test_term(self):
        c = zmqpy.Context()
        c.term()
        self.assert_(c.closed)

    def test_fail_init(self):
        self.assertRaisesErrno(zmqpy.EINVAL, zmqpy.Context, 0)

    def test_instance_state(self):
        ctx = zmqpy.Context()
        c2 = zmqpy.Context(iothreads=2)
        self.assertTrue(ctx.iothreads)
        c2.term()
        self.assertTrue(ctx.closed)
        c3 = zmqpy.Context()
        c4 = zmqpy.Context()
        self.assertFalse(c3.closed)
        self.assertFalse(c4.closed)
        c4.term()
        self.assertTrue(ctx.closed)

    def test_term_hang(self):
        rep, req = self.create_bound_pair(zmqpy.XREP, zmqpy.XREQ)
        req.setsockopt(zmqpy.LINGER, 0)
        req.send(asbytes('hello'), copy=False)
        req.close()
        rep.close()
        self.context.term()

    def test_many_sockets(self):
        """opening and closing many sockets shouldn't cause problems"""
        ctx = self.context
        for i in range(16):
            sockets = [ ctx.socket(zmqpy.REP) for i in range(65) ]
            [ s.close() for s in sockets ]
            # give the reaper a chance
            time.sleep(1e-2)
        ctx.term()
        for s in sockets:
            self.assertTrue(s.closed)

    def test_term_close(self):
        """Context.term should close sockets"""
        sockets = [ self.context.socket(zmqpy.REP) for i in range(65) ]
        # close half of the sockets
        [ s.close() for s in sockets[::2] ]
        self.context.term()
        for s in sockets:
            self.assertTrue(s.closed)

    def test_set_linger(self):
        ctx = zmqpy.Context()
        self.assertEquals(ctx.linger, 1)

        ctx.set_linger(10)
        self.assertEquals(ctx.linger, 10)

if __name__ == "__main__":
    unittest.main()



# coding: utf-8

from ._cffi import C, constant_names, zmq_version

names = None
pynames = []

_constants = {}

for cname in constant_names:
    pyname = cname.split('_', 1)[-1]
    pynames.append(pyname)
    _constants[pyname] = getattr(C, cname)

globals().update(_constants)

if zmq_version == 2:
    DONTWAIT = NOBLOCK
    pynames.append('DONTWAIT')
    globals()['DONTWAIT'] = DONTWAIT
else:
    NOBLOCK = DONTWAIT
    pynames.append('NOBLOCK')
    globals()['NOBLOCK'] = NOBLOCK

uint64_opts = int64_opts = binary_opts = int_opts = []

if zmq_version == 2:
    uint64_opts = [HWM, AFFINITY, SNDBUF, RCVBUF]

    int64_opts =  [SWAP, RECOVERY_IVL, RECOVERY_IVL_MSEC,
                   MCAST_LOOP, RATE, RCVMORE]

    binary_opts = [IDENTITY, SUBSCRIBE, UNSUBSCRIBE]

    int_opts =    [RCVTIMEO, SNDTIMEO, LINGER, RECONNECT_IVL,
                   RECONNECT_IVL_MAX, BACKLOG, FD, EVENTS, TYPE]

    DONTWAIT = NOBLOCK
else:
    uint64_opts = [AFFINITY, SNDBUF, RCVBUF]

    int64_opts =  [RECOVERY_IVL]

    binary_opts = [IDENTITY, SUBSCRIBE, UNSUBSCRIBE]

    int_opts =    [RCVHWM, SNDHWM, RCVTIMEO, SNDTIMEO,
                   LINGER, RECONNECT_IVL, RECONNECT_IVL_MAX,
                   BACKLOG, RATE, RCVMORE, FD, EVENTS, TYPE]

    NOBLOCK = DONTWAIT

# compatibility with default core constants

int_sockopts = []
int_sockopts.extend(int_opts)

int64_sockopts = []
int64_sockopts.extend(int64_opts)
int64_sockopts.extend(uint64_opts)

bytes_sockopts = []
bytes_sockopts.extend(binary_opts)

pynames.extend([
    'binary_opts',
    'int_opts',
    'int64_opts',
    'uint64_opts',
    'bytes_sockopts',
])

__all__ = pynames



#
# Copyright 2009 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""An I/O event loop for non-blocking sockets.

Typical applications will use a single `IOLoop` object, in the
`IOLoop.instance` singleton.  The `IOLoop.start` method should usually
be called at the end of the ``main()`` function.  Atypical applications may
use more than one `IOLoop`, such as one `IOLoop` per thread, or per `unittest`
case.

In addition to I/O events, the `IOLoop` can also schedule time-based events.
`IOLoop.add_timeout` is a non-blocking alternative to `time.sleep`.
"""

from __future__ import absolute_import, division, with_statement

import datetime
import errno
import heapq
import os
import sys
import logging
import threading
import time
import traceback

try:
    import thread
except ImportError:
    # py3k removed thread
    thread_get_ident = lambda : threading.current_thread().ident
else:
    thread_get_ident = thread.get_ident

from zmqpy.eventloop import stack_context

try:
    import signal
except ImportError:
    signal = None

from zmqpy.eventloop.platform.auto import set_close_exec, Waker

from zmqpy import (
    Poller,
    POLLIN, POLLOUT, POLLERR,
    ZMQError #, ETERM,
)

if sys.version_info[0] >= 3:
    # all ints are long in py3k
    long = int

class IOLoop(object):
    """A level-triggered I/O loop.

    We use the zmq Poller for polling events.

    Example usage for a simple TCP server::

        import errno
        import functools
        import ioloop
        import socket

        def connection_ready(sock, fd, events):
            while True:
                try:
                    connection, address = sock.accept()
                except socket.error, e:
                    if e.args[0] not in (errno.EWOULDBLOCK, errno.EAGAIN):
                        raise
                    return
                connection.setblocking(0)
                handle_connection(connection, address)

        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.setblocking(0)
        sock.bind(("", port))
        sock.listen(128)

        io_loop = ioloop.IOLoop.instance()
        callback = functools.partial(connection_ready, sock)
        io_loop.add_handler(sock.fileno(), callback, io_loop.READ)
        io_loop.start()

    """
    # Constants from the epoll module
    _EPOLLIN = 0x001
    _EPOLLPRI = 0x002
    _EPOLLOUT = 0x004
    _EPOLLERR = 0x008
    _EPOLLHUP = 0x010
    _EPOLLRDHUP = 0x2000
    _EPOLLONESHOT = (1 << 30)
    _EPOLLET = (1 << 31)

    # Our events map exactly to the epoll events
    NONE = 0
    READ = _EPOLLIN
    WRITE = _EPOLLOUT
    ERROR = _EPOLLERR | _EPOLLHUP

    # Global lock for creating global IOLoop instance
    _instance_lock = threading.Lock()

    def __init__(self, impl=None):
        self._impl = impl or _poll()
        if hasattr(self._impl, 'fileno'):
            set_close_exec(self._impl.fileno())
        self._handlers = {}
        self._events = {}
        self._callbacks = []
        self._callback_lock = threading.Lock()
        self._timeouts = []
        self._running = False
        self._stopped = False
        self._thread_ident = None
        self._blocking_signal_threshold = None

        # Create a pipe that we send bogus data to when we want to wake
        # the I/O loop when it is idle
        self._waker = Waker()
        self.add_handler(self._waker.fileno(),
                         lambda fd, events: self._waker.consume(),
                         self.READ)

    @staticmethod
    def instance():
        """Returns a global IOLoop instance.

        Most single-threaded applications have a single, global IOLoop.
        Use this method instead of passing around IOLoop instances
        throughout your code.

        A common pattern for classes that depend on IOLoops is to use
        a default argument to enable programs with multiple IOLoops
        but not require the argument for simpler applications::

            class MyClass(object):
                def __init__(self, io_loop=None):
                    self.io_loop = io_loop or IOLoop.instance()
        """
        if not hasattr(IOLoop, "_instance"):
            with IOLoop._instance_lock:
                if not hasattr(IOLoop, "_instance"):
                    # New instance after double check
                    IOLoop._instance = IOLoop()
        return IOLoop._instance

    @staticmethod
    def initialized():
        """Returns true if the singleton instance has been created."""
        return hasattr(IOLoop, "_instance")

    def install(self):
        """Installs this IOloop object as the singleton instance.

        This is normally not necessary as `instance()` will create
        an IOLoop on demand, but you may want to call `install` to use
        a custom subclass of IOLoop.
        """
        assert not IOLoop.initialized()
        IOLoop._instance = self

    def close(self, all_fds=False):
        """Closes the IOLoop, freeing any resources used.

        If ``all_fds`` is true, all file descriptors registered on the
        IOLoop will be closed (not just the ones created by the IOLoop itself).

        Many applications will only use a single IOLoop that runs for the
        entire lifetime of the process.  In that case closing the IOLoop
        is not necessary since everything will be cleaned up when the
        process exits.  `IOLoop.close` is provided mainly for scenarios
        such as unit tests, which create and destroy a large number of
        IOLoops.

        An IOLoop must be completely stopped before it can be closed.  This
        means that `IOLoop.stop()` must be called *and* `IOLoop.start()` must
        be allowed to return before attempting to call `IOLoop.close()`.
        Therefore the call to `close` will usually appear just after
        the call to `start` rather than near the call to `stop`.
        """
        self.remove_handler(self._waker.fileno())
        if all_fds:
            for fd in self._handlers.keys()[:]:
                try:
                    if hasattr(fd, 'close'):
                        fd.close()
                    else:
                        os.close(fd)
                except Exception:
                    logging.debug("error closing fd %s", fd, exc_info=True)
        self._waker.close()
        self._impl.close()

    def add_handler(self, fd, handler, events):
        """Registers the given handler to receive the given events for fd."""
        self._handlers[fd] = stack_context.wrap(handler)
        self._impl.register(fd, events | self.ERROR)

    def update_handler(self, fd, events):
        """Changes the events we listen for fd."""
        self._impl.modify(fd, events | self.ERROR)

    def remove_handler(self, fd):
        """Stop listening for events on fd."""
        self._handlers.pop(fd, None)
        self._events.pop(fd, None)
        try:
            self._impl.unregister(fd)
        except (OSError, IOError):
            logging.debug("Error deleting fd from IOLoop", exc_info=True)

    def set_blocking_signal_threshold(self, seconds, action):
        """Sends a signal if the ioloop is blocked for more than s seconds.

        Pass seconds=None to disable.  Requires python 2.6 on a unixy
        platform.

        The action parameter is a python signal handler.  Read the
        documentation for the python 'signal' module for more information.
        If action is None, the process will be killed if it is blocked for
        too long.
        """
        if not hasattr(signal, "setitimer"):
            logging.error("set_blocking_signal_threshold requires a signal module "
                       "with the setitimer method")
            return
        self._blocking_signal_threshold = seconds
        if seconds is not None:
            signal.signal(signal.SIGALRM,
                          action if action is not None else signal.SIG_DFL)

    def set_blocking_log_threshold(self, seconds):
        """Logs a stack trace if the ioloop is blocked for more than s seconds.
        Equivalent to set_blocking_signal_threshold(seconds, self.log_stack)
        """
        self.set_blocking_signal_threshold(seconds, self.log_stack)

    def log_stack(self, signal, frame):
        """Signal handler to log the stack trace of the current thread.

        For use with set_blocking_signal_threshold.
        """
        logging.warning('IOLoop blocked for %f seconds in\n%s',
                        self._blocking_signal_threshold,
                        ''.join(traceback.format_stack(frame)))

    def start(self):
        """Starts the I/O loop.

        The loop will run until one of the I/O handlers calls stop(), which
        will make the loop stop after the current event iteration completes.
        """
        if self._stopped:
            self._stopped = False
            return
        self._thread_ident = thread_get_ident()
        self._running = True
        while True:
            poll_timeout = 3600.0

            # Prevent IO event starvation by delaying new callbacks
            # to the next iteration of the event loop.
            with self._callback_lock:
                callbacks = self._callbacks
                self._callbacks = []
            for callback in callbacks:
                self._run_callback(callback)

            if self._timeouts:
                now = time.time()
                while self._timeouts:
                    if self._timeouts[0].callback is None:
                        # the timeout was cancelled
                        heapq.heappop(self._timeouts)
                    elif self._timeouts[0].deadline <= now:
                        timeout = heapq.heappop(self._timeouts)
                        self._run_callback(timeout.callback)
                    else:
                        seconds = self._timeouts[0].deadline - now
                        poll_timeout = min(seconds, poll_timeout)
                        break

            if self._callbacks:
                # If any callbacks or timeouts called add_callback,
                # we don't want to wait in poll() before we run them.
                poll_timeout = 0.0

            if not self._running:
                break

            if self._blocking_signal_threshold is not None:
                # clear alarm so it doesn't fire while poll is waiting for
                # events.
                signal.setitimer(signal.ITIMER_REAL, 0, 0)

            try:
                event_pairs = self._impl.poll(poll_timeout)
            except Exception as e:
                # Depending on python version and IOLoop implementation,
                # different exception types may be thrown and there are
                # two ways EINTR might be signaled:
                # * e.errno == errno.EINTR
                # * e.args is like (errno.EINTR, 'Interrupted system call')
                if (getattr(e, 'errno', None) == errno.EINTR or
                    (isinstance(getattr(e, 'args', None), tuple) and
                     len(e.args) == 2 and e.args[0] == errno.EINTR)):
                    continue
                # elif getattr(e, 'errno', None) == ETERM:
                #     # This happens when the zmq Context is closed; we should just exit.
                #     self._running = False
                #     self._stopped = True
                #     break
                else:
                    raise

            if self._blocking_signal_threshold is not None:
                signal.setitimer(signal.ITIMER_REAL,
                                 self._blocking_signal_threshold, 0)

            # Pop one fd at a time from the set of pending fds and run
            # its handler. Since that handler may perform actions on
            # other file descriptors, there may be reentrant calls to
            # this IOLoop that update self._events
            self._events.update(event_pairs)
            while self._events:
                fd, events = self._events.popitem()
                try:
                    self._handlers[fd](fd, events)
                except (OSError, IOError) as e:
                    if e.args[0] == errno.EPIPE:
                        # Happens when the client closes the connection
                        pass
                    else:
                        logging.error("Exception in I/O handler for fd %s",
                                      fd, exc_info=True)
                except Exception:
                    logging.error("Exception in I/O handler for fd %s",
                                  fd, exc_info=True)
        # reset the stopped flag so another start/stop pair can be issued
        self._stopped = False
        if self._blocking_signal_threshold is not None:
            signal.setitimer(signal.ITIMER_REAL, 0, 0)

    def stop(self):
        """Stop the loop after the current event loop iteration is complete.
        If the event loop is not currently running, the next call to start()
        will return immediately.

        To use asynchronous methods from otherwise-synchronous code (such as
        unit tests), you can start and stop the event loop like this::

          ioloop = IOLoop()
          async_method(ioloop=ioloop, callback=ioloop.stop)
          ioloop.start()

        ioloop.start() will return after async_method has run its callback,
        whether that callback was invoked before or after ioloop.start.

        Note that even after `stop` has been called, the IOLoop is not
        completely stopped until `IOLoop.start` has also returned.
        """
        self._running = False
        self._stopped = True
        self._waker.wake()

    def running(self):
        """Returns true if this IOLoop is currently running."""
        return self._running

    def add_timeout(self, deadline, callback):
        """Calls the given callback at the time deadline from the I/O loop.

        Returns a handle that may be passed to remove_timeout to cancel.

        ``deadline`` may be a number denoting a unix timestamp (as returned
        by ``time.time()`` or a ``datetime.timedelta`` object for a deadline
        relative to the current time.

        Note that it is not safe to call `add_timeout` from other threads.
        Instead, you must use `add_callback` to transfer control to the
        IOLoop's thread, and then call `add_timeout` from there.
        """
        timeout = _Timeout(deadline, stack_context.wrap(callback))
        heapq.heappush(self._timeouts, timeout)
        return timeout

    def remove_timeout(self, timeout):
        """Cancels a pending timeout.

        The argument is a handle as returned by add_timeout.
        """
        # Removing from a heap is complicated, so just leave the defunct
        # timeout object in the queue (see discussion in
        # http://docs.python.org/library/heapq.html).
        # If this turns out to be a problem, we could add a garbage
        # collection pass whenever there are too many dead timeouts.
        timeout.callback = None

    def add_callback(self, callback):
        """Calls the given callback on the next I/O loop iteration.

        It is safe to call this method from any thread at any time.
        Note that this is the *only* method in IOLoop that makes this
        guarantee; all other interaction with the IOLoop must be done
        from that IOLoop's thread.  add_callback() may be used to transfer
        control from other threads to the IOLoop's thread.
        """
        with self._callback_lock:
            list_empty = not self._callbacks
            self._callbacks.append(stack_context.wrap(callback))
        if list_empty and thread_get_ident() != self._thread_ident:
            # If we're in the IOLoop's thread, we know it's not currently
            # polling.  If we're not, and we added the first callback to an
            # empty list, we may need to wake it up (it may wake up on its
            # own, but an occasional extra wake is harmless).  Waking
            # up a polling IOLoop is relatively expensive, so we try to
            # avoid it when we can.
            self._waker.wake()

    def _run_callback(self, callback):
        try:
            callback()
        except Exception:
            self.handle_callback_exception(callback)

    def handle_callback_exception(self, callback):
        """This method is called whenever a callback run by the IOLoop
        throws an exception.

        By default simply logs the exception as an error.  Subclasses
        may override this method to customize reporting of exceptions.

        The exception itself is not passed explicitly, but is available
        in sys.exc_info.
        """
        logging.error("Exception in callback %r", callback, exc_info=True)


class _Timeout(object):
    """An IOLoop timeout, a UNIX timestamp and a callback"""

    # Reduce memory overhead when there are lots of pending callbacks
    __slots__ = ['deadline', 'callback']

    def __init__(self, deadline, callback):
        if isinstance(deadline, (int, long, float)):
            self.deadline = deadline
        elif isinstance(deadline, datetime.timedelta):
            self.deadline = time.time() + _Timeout.timedelta_to_seconds(deadline)
        else:
            raise TypeError("Unsupported deadline %r" % deadline)
        self.callback = callback

    @staticmethod
    def timedelta_to_seconds(td):
        """Equivalent to td.total_seconds() (introduced in python 2.7)."""
        return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10 ** 6) / float(10 ** 6)

    # Comparison methods to sort by deadline, with object id as a tiebreaker
    # to guarantee a consistent ordering.  The heapq module uses __le__
    # in python2.5, and __lt__ in 2.6+ (sort() and most other comparisons
    # use __lt__).
    def __lt__(self, other):
        return ((self.deadline, id(self)) <
                (other.deadline, id(other)))

    def __le__(self, other):
        return ((self.deadline, id(self)) <=
                (other.deadline, id(other)))


class PeriodicCallback(object):
    """Schedules the given callback to be called periodically.

    The callback is called every callback_time milliseconds.

    `start` must be called after the PeriodicCallback is created.
    """
    def __init__(self, callback, callback_time, io_loop=None):
        self.callback = callback
        self.callback_time = callback_time
        self.io_loop = io_loop or IOLoop.instance()
        self._running = False
        self._timeout = None

    def start(self):
        """Starts the timer."""
        self._running = True
        self._next_timeout = time.time()
        self._schedule_next()

    def stop(self):
        """Stops the timer."""
        self._running = False
        if self._timeout is not None:
            self.io_loop.remove_timeout(self._timeout)
            self._timeout = None

    def _run(self):
        if not self._running:
            return
        try:
            self.callback()
        except Exception:
            logging.error("Error in periodic callback", exc_info=True)
        self._schedule_next()

    def _schedule_next(self):
        if self._running:
            current_time = time.time()
            while self._next_timeout <= current_time:
                self._next_timeout += self.callback_time / 1000.0
            self._timeout = self.io_loop.add_timeout(self._next_timeout, self._run)

class DelayedCallback(PeriodicCallback):
    """Schedules the given callback to be called once.

    The callback is called once, after callback_time milliseconds.

    `start` must be called after the DelayedCallback is created.

    The timeout is calculated from when `start` is called.
    """

    def start(self):
        """Starts the timer."""
        self._running = True
        self._firstrun = True
        self._next_timeout = time.time() + self.callback_time / 1000.0
        self.io_loop.add_timeout(self._next_timeout, self._run)

    def _run(self):
        if not self._running: return
        self._running = False
        try:
            self.callback()
        except Exception:
            logging.error("Error in delayed callback", exc_info=True)


class ZMQPoller(object):
    """A poller that can be used in the tornado IOLoop.

    This simply wraps a regular zmq.Poller, scaling the timeout
    by 1000, so that it is in seconds rather than milliseconds.
    """

    def __init__(self):
        self._poller = Poller()

    @staticmethod
    def _map_events(events):
        """translate IOLoop.READ/WRITE/ERROR event masks into zmq.POLLIN/OUT/ERR"""
        z_events = 0
        if events & IOLoop.READ:
            z_events |= POLLIN
        if events & IOLoop.WRITE:
            z_events |= POLLOUT
        if events & IOLoop.ERROR:
            z_events |= POLLERR
        return z_events

    @staticmethod
    def _remap_events(z_events):
        """translate zmq.POLLIN/OUT/ERR event masks into IOLoop.READ/WRITE/ERROR"""
        events = 0
        if z_events & POLLIN:
            events |= IOLoop.READ
        if z_events & POLLOUT:
            events |= IOLoop.WRITE
        if z_events & POLLERR:
            events |= IOLoop.ERROR
        return events

    def register(self, fd, events):
        return self._poller.register(fd, self._map_events(events))

    def modify(self, fd, events):
        return self._poller.modify(fd, self._map_events(events))

    def unregister(self, fd):
        return self._poller.unregister(fd)

    def poll(self, timeout):
        """poll in seconds rather than milliseconds.

        Event masks will be IOLoop.READ/WRITE/ERROR
        """
        z_events = self._poller.poll(1000*timeout)
        return [ (fd,self._remap_events(evt)) for (fd,evt) in z_events ]

    def close(self):
        pass

_poll = ZMQPoller

def install():
    """set the tornado IOLoop instance with the pyzmq IOLoop.

    After calling this function, tornado's IOLoop.instance() and pyzmq's
    IOLoop.instance() will return the same object.

    An assertion error will be raised if tornado's IOLoop has been initialized
    prior to calling this function.
    """
    from tornado import ioloop
    # check if tornado's IOLoop is already initialized to something other
    # than the pyzmq IOLoop instance:
    assert (not ioloop.IOLoop.initialized()) or \
        ioloop.IOLoop.instance() is IOLoop.instance(), "tornado IOLoop already initialized"
    # register as instance with tornado.ioloop
    ioloop.IOLoop._instance = IOLoop.instance()




"""Declare basic string types unambiguously for various Python versions.

Authors
-------
* MinRK
"""

#
#    Copyright (c) 2010 Min Ragan-Kelley, Brian Granger
#
#    This file is part of pyzmq.
#
#    pyzmq is free software; you can redistribute it and/or modify it under
#    the terms of the Lesser GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    pyzmq is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    Lesser GNU General Public License for more details.
#
#    You should have received a copy of the Lesser GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import sys

major,mior = sys.version_info[:2]
if major >= 3:
    bytes = bytes
    unicode = str
    basestring = (bytes, unicode)
    asbytes = lambda s: s if isinstance(s, bytes) else unicode(s).encode('utf8')

elif major == 2:
    unicode = unicode
    bytes = str
    basestring = basestring
    asbytes = str

__all__ = ['asbytes', 'bytes', 'unicode', 'basestring']




from setuptools import setup

setup(name='zmqpy',
      version='0.3.3',
      description='Python cffi-based ZeroMQ bindings',
      author='Felipe Cruz',
      author_email='felipecruz@loogica.net',
      url='https://github.com/felipecruz/zmqpy',
      install_requires=['cffi>=0.3'],
      packages=['zmqpy', 'zmqpy/tests/', 'zmqpy/utils/', 'zmqpy/eventloop',
                'zmqpy/tests/pyzmq_tests/'])



import pytest

def test_zmq_get_version():
    from zmqpy._cffi import get_version

    version = get_version()

    assert version[0] in (2, 3)

def test_zmq_init():
    from zmqpy._cffi import C

    ctx = C.zmq_init(1)

    assert ctx
    ret = C.zmq_term(ctx)

def test_zmq_term():
    from zmqpy._cffi import C

    ctx = C.zmq_init(1)
    ret = C.zmq_term(ctx)

    assert ret == 0

def test_zmq_socket():
    from zmqpy._cffi import C
    from zmqpy.constants import PUSH

    ctx = C.zmq_init(1)
    socket = C.zmq_socket(ctx, PUSH)

    assert socket

def test_zmq_socket_close():
    from zmqpy._cffi import C
    from zmqpy.constants import PUSH

    ctx = C.zmq_init(1)
    socket = C.zmq_socket(ctx, PUSH)

    ret = C.zmq_close(socket)

    assert ret == 0

def test_zmq_setsockopt():
    from zmqpy._cffi import C, ffi
    from zmqpy.constants import PUSH, IDENTITY

    ctx = C.zmq_init(1)
    socket = C.zmq_socket(ctx, PUSH)

    identity = ffi.new('char[3]', 'zmq')
    ret = C.zmq_setsockopt(socket, IDENTITY, ffi.cast('void*', identity), 3)
    assert ret == 0

def test_zmq_getsockopt():
    from zmqpy._cffi import C, ffi
    from zmqpy.constants import PUSH, IDENTITY

    ctx = C.zmq_init(1)
    socket = C.zmq_socket(ctx, PUSH)

    identity = ffi.new('char[]', 'zmq')
    ret = C.zmq_setsockopt(socket, IDENTITY, ffi.cast('void*', identity), 3)
    assert ret == 0

    option_len = ffi.new('size_t*', 3)
    option = ffi.new('char*')
    ret = C.zmq_getsockopt(socket,
                           IDENTITY,
                           ffi.cast('void*', option),
                           option_len)

    assert ret == 0
    assert ffi.string(ffi.cast('char*', option))[0] == "z"
    assert ffi.string(ffi.cast('char*', option))[1] == "m"
    assert ffi.string(ffi.cast('char*', option))[2] == "q"

def test_zmq_bind_connect():
    from zmqpy.constants import PAIR

    ctx = C.zmq_init(1)
    socket1 = C.zmq_socket(ctx, PAIR)
    r1 = C.zmq_bind(socket1, 'tcp://*:5555')

    assert r1 == 0

def test_zmq_bind_connect():
    from zmqpy._cffi import C, ffi
    from zmqpy.constants import PAIR

    ctx = C.zmq_init(1)

    socket1 = C.zmq_socket(ctx, PAIR)
    socket2 = C.zmq_socket(ctx, PAIR)

    r1 = C.zmq_bind(socket1, 'tcp://*:5555')
    r2 = C.zmq_connect(socket2, 'tcp://127.0.0.1:5555')

    assert r1 == 0
    assert r2 == 0

def test_zmq_msg_init():
    from zmqpy._cffi import C, ffi

    zmq_msg = ffi.new('zmq_msg_t*')

    assert zmq_msg

    C.zmq_msg_init(zmq_msg)

    assert zmq_msg

def test_zmq_msg_close():
    from zmqpy._cffi import C, ffi

    zmq_msg = ffi.new('zmq_msg_t*')
    C.zmq_msg_init(zmq_msg)
    ret = C.zmq_msg_close(zmq_msg)

    assert ret == 0

def test_zmq_msg_init_size():
    from zmqpy._cffi import C, ffi

    zmq_msg = ffi.new('zmq_msg_t*')

    assert zmq_msg

    C.zmq_msg_init_size(zmq_msg, 10)

    assert zmq_msg

def test_zmq_msg_init_data():
    from zmqpy._cffi import C, ffi

    zmq_msg = ffi.new('zmq_msg_t*')
    assert zmq_msg

    message = ffi.new('char[5]', 'Hello')
    C.zmq_msg_init_data(zmq_msg, ffi.cast('void*', message), 5, ffi.NULL,
                                                                ffi.NULL)

    assert zmq_msg

def test_zmq_msg_close():
    from zmqpy._cffi import C, ffi

    zmq_msg = ffi.new('zmq_msg_t*')
    assert zmq_msg

    message = ffi.new('char[]', 'Hello')
    C.zmq_msg_init_data(zmq_msg, ffi.cast('void*', message), 5, ffi.NULL,
                                                                ffi.NULL)

    assert zmq_msg

    ret = C.zmq_msg_close(zmq_msg)
    assert ret == 0

def test_zmq_msg_data():
    from zmqpy._cffi import C, ffi

    zmq_msg = ffi.new('zmq_msg_t*')
    assert zmq_msg

    message = ffi.new('char[]', 'Hello')
    C.zmq_msg_init_data(zmq_msg, ffi.cast('void*', message), 5, ffi.NULL,
                                                                ffi.NULL)

    assert zmq_msg

    data = C.zmq_msg_data(zmq_msg)
    assert ffi.string(ffi.cast("char*", data)) == 'Hello'


def test_zmq_send():
    from zmqpy.constants import PAIR, NOBLOCK
    from zmqpy._cffi import C, ffi, zmq_version

    zmq_msg = ffi.new('zmq_msg_t*')

    message = ffi.new('char[5]', 'Hello')
    C.zmq_msg_init_data(zmq_msg, ffi.cast('void*', message), 5, ffi.NULL,
                                                                ffi.NULL)

    ctx = C.zmq_init(1)

    socket1 = C.zmq_socket(ctx, PAIR)
    socket2 = C.zmq_socket(ctx, PAIR)

    r1 = C.zmq_bind(socket1, 'tcp://*:5555')
    r2 = C.zmq_connect(socket2, 'tcp://127.0.0.1:5555')

    if zmq_version == 2:
        ret = C.zmq_send(socket2, zmq_msg, 0)
        assert ret == 0
    else:
        ret = C.zmq_sendmsg(socket2, zmq_msg, 0)
        assert ret == 5

    assert C.zmq_close(socket1) == 0
    assert C.zmq_close(socket2) == 0
    assert C.zmq_term(ctx) == 0

def test_zmq_recv():
    from zmqpy.constants import REQ, REP, NOBLOCK
    from zmqpy._cffi import C, ffi, zmq_version

    ctx = C.zmq_init(1)

    sender = C.zmq_socket(ctx, REQ)
    receiver = C.zmq_socket(ctx, REP)

    r1 = C.zmq_bind(receiver, 'tcp://*:3333')
    r2 = C.zmq_connect(sender, 'tcp://127.0.0.1:3333')

    zmq_msg = ffi.new('zmq_msg_t*')
    message = ffi.new('char[5]', 'Hello')

    C.zmq_msg_init_data(zmq_msg,
                        ffi.cast('void*', message),
                        ffi.cast('size_t', 5),
                        ffi.NULL,
                        ffi.NULL)

    zmq_msg2 = ffi.new('zmq_msg_t*')
    C.zmq_msg_init(zmq_msg2)

    if zmq_version == 2:
        ret = C.zmq_send(sender, zmq_msg, 0)
        ret2 = C.zmq_recv(receiver, zmq_msg2, 0)
        assert ret == ret2 == 0
    else:
        ret = C.zmq_sendmsg(sender, zmq_msg, 0)
        ret2 = C.zmq_recvmsg(receiver, zmq_msg2, 0)
        assert ret == ret2 == 5

    assert 5 == C.zmq_msg_size(zmq_msg2)
    assert "Hello" == ffi.buffer(C.zmq_msg_data(zmq_msg2),
                                 C.zmq_msg_size(zmq_msg2))[:]

    assert C.zmq_close(sender) == 0
    assert C.zmq_close(receiver) == 0
    assert C.zmq_term(ctx) == 0

def test_zmq_poll():
    from zmqpy.constants import REQ, REP, NOBLOCK, POLLIN, POLLOUT
    from zmqpy._cffi import C, ffi, zmq_version

    ctx = C.zmq_init(1)

    sender = C.zmq_socket(ctx, REQ)
    receiver = C.zmq_socket(ctx, REP)

    r1 = C.zmq_bind(receiver, 'tcp://*:3333')
    r2 = C.zmq_connect(sender, 'tcp://127.0.0.1:3333')

    zmq_msg = ffi.new('zmq_msg_t*')
    message = ffi.new('char[5]', 'Hello')

    C.zmq_msg_init_data(zmq_msg,
                        ffi.cast('void*', message),
                        ffi.cast('size_t', 5),
                        ffi.NULL,
                        ffi.NULL)

    receiver_pollitem = ffi.new('zmq_pollitem_t*')
    receiver_pollitem.socket = receiver
    receiver_pollitem.fd = 0
    receiver_pollitem.events = POLLIN | POLLOUT
    receiver_pollitem.revents = 0

    ret = C.zmq_poll(ffi.NULL, 0, 0)
    assert ret == 0

    ret = C.zmq_poll(receiver_pollitem, 1, 0)
    assert ret == 0

    if zmq_version == 2:
        ret = C.zmq_send(sender, zmq_msg, 0)
        assert ret == 0
    else:
        ret = C.zmq_sendmsg(sender, zmq_msg, 0)
        assert ret == 5


    assert C.zmq_msg_close(zmq_msg) == 0

    import time
    time.sleep(0.2)

    ret = C.zmq_poll(receiver_pollitem, 1, 0)
    assert ret == 1

    assert int(receiver_pollitem.revents) & POLLIN
    assert not int(receiver_pollitem.revents) & POLLOUT

    zmq_msg2 = ffi.new('zmq_msg_t*')
    C.zmq_msg_init(zmq_msg2)

    if zmq_version == 2:
        ret_recv = C.zmq_recv(receiver, zmq_msg2, 0)
        assert ret_recv == 0
    else:
        ret_recv = C.zmq_recvmsg(receiver, zmq_msg2, 0)
        assert ret_recv == 5

    assert 5 == C.zmq_msg_size(zmq_msg2)
    assert "Hello" == ffi.buffer(C.zmq_msg_data(zmq_msg2),
                                 C.zmq_msg_size(zmq_msg2))[:]

    #assert C.zmq_msg_close(zmq_msg2) == 0

    sender_pollitem = ffi.new('zmq_pollitem_t*')
    sender_pollitem.socket = sender
    sender_pollitem.fd = 0
    sender_pollitem.events = POLLIN | POLLOUT
    sender_pollitem.revents = 0

    ret = C.zmq_poll(sender_pollitem, 1, 0)
    assert ret == 0

    zmq_msg_again = ffi.new('zmq_msg_t*')
    message_again = ffi.new('char[11]', 'Hello Again')

    C.zmq_msg_init_data(zmq_msg_again,
                        ffi.cast('void*', message_again),
                        ffi.cast('size_t', 11),
                        ffi.NULL,
                        ffi.NULL)

    if zmq_version == 2:
        ret_send = C.zmq_send(receiver, zmq_msg_again, 0)
        assert ret_send == 0
    else:
        ret_send = C.zmq_sendmsg(receiver, zmq_msg_again, 0)
        assert ret_send == 11

    import time
    time.sleep(0.2)

    ret = C.zmq_poll(sender_pollitem, 1, 0)

    assert ret >= 0
    assert int(sender_pollitem.revents) & POLLIN

    if zmq_version == 2:
        ret_recv = C.zmq_recv(sender, zmq_msg2, 0)
        assert ret_recv == 0
    else:
        ret_recv = C.zmq_recvmsg(sender, zmq_msg2, 0)
        assert ret_recv == 11

    assert 11 == C.zmq_msg_size(zmq_msg2)
    assert "Hello Again" == ffi.buffer(C.zmq_msg_data(zmq_msg2), int(C.zmq_msg_size(zmq_msg2)))[:]
    assert C.zmq_close(sender) == 0
    assert C.zmq_close(receiver) == 0
    assert C.zmq_term(ctx) == 0



"""Priority based json library imports.

Use jsonapi.loads() and jsonapi.dumps() for guaranteed symmetry.

Priority: jsonlib2 > jsonlib > simplejson > json

Ensures bytes instead of unicode on either side of serialization.

Authors
-------
* MinRK
* Brian Granger
"""

#
#    Copyright (c) 2010 Min Ragan-Kelley, Brian Granger
#
#    This file is part of pyzmq.
#
#    pyzmq is free software; you can redistribute it and/or modify it under
#    the terms of the Lesser GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    pyzmq is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    Lesser GNU General Public License for more details.
#
#    You should have received a copy of the Lesser GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

from zmqpy.utils.strtypes import bytes, unicode
# priority: jsonlib2 > jsonlib > simplejson > json

jsonmod = None

try:
    import jsonlib2 as jsonmod
except ImportError:
    try:
        import jsonlib as jsonmod
    except ImportError:
        try:
            import simplejson as jsonmod
        except ImportError:
            try:
                import json as jsonmod
            except ImportError:
                pass

def _squash_unicode(s):
    if isinstance(s, unicode):
        return s.encode('utf8')
    else:
        return s

def jsonlib_dumps(o,**kwargs):
    """This one is separate because jsonlib doesn't allow specifying separators.
    See jsonlib.dumps for details on kwargs.
    """
    return _squash_unicode(jsonmod.dumps(o,**kwargs))

def dumps(o, **kwargs):
    """Serialize object to JSON str.
    See %s.dumps for details on kwargs.
    """%jsonmod
    
    return _squash_unicode(jsonmod.dumps(o, separators=(',',':'),**kwargs))

def loads(s,**kwargs):
    """Load object from JSON str.
    See %s.loads for details on kwargs.
    """%jsonmod
    if str is unicode and isinstance(s, bytes):
        s = s.decode('utf8')
    return jsonmod.loads(s,**kwargs)

if jsonmod is not None and jsonmod.__name__== 'jsonlib':
    dumps = jsonlib_dumps

__all__ = ['jsonmod', 'dumps', 'dumps']



#-----------------------------------------------------------------------------
#  Copyright (c) 2010-2012 Brian Granger, Min Ragan-Kelley
#
#  This file is part of pyzmq
#
#  Distributed under the terms of the New BSD License.  The full license is in
#  the file COPYING.BSD, distributed as part of this software.
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

import time
import os
import threading

import zmqpy as zmq
from .__init__ import BaseZMQTestCase
from zmqpy.eventloop import ioloop


#-----------------------------------------------------------------------------
# Tests
#-----------------------------------------------------------------------------
def printer():
    os.system("say hello")
    raise Exception
    print (time.time())

class Delay(threading.Thread):
    def __init__(self, f, delay=1):
        self.f=f
        self.delay=delay
        self.aborted=False
        self.cond=threading.Condition()
        super(Delay, self).__init__()

    def run(self):
        self.cond.acquire()
        self.cond.wait(self.delay)
        self.cond.release()
        if not self.aborted:
            self.f()

    def abort(self):
        self.aborted=True
        self.cond.acquire()
        self.cond.notify()
        self.cond.release()

class TestIOLoop(BaseZMQTestCase):

    def test_simple(self):
        """simple IOLoop creation test"""
        loop = ioloop.IOLoop()
        dc = ioloop.DelayedCallback(loop.stop, 200, loop)
        pc = ioloop.DelayedCallback(lambda : None, 10, loop)
        pc.start()
        dc.start()
        t = Delay(loop.stop,1)
        t.start()
        loop.start()
        if t.isAlive():
            t.abort()
        else:
            self.fail("IOLoop failed to exit")

    def test_timeout_compare(self):
        """test timeout comparisons"""
        t = ioloop._Timeout(1,2)
        t2 = ioloop._Timeout(1,3)
        self.assertEquals(t < t2, id(t) < id(t2))
        t2 = ioloop._Timeout(2,1)
        self.assertTrue(t < t2)

    def test_poller_events(self):
        """Tornado poller implementation maps events correctly"""
        req,rep = self.create_bound_pair(zmq.REQ, zmq.REP)
        poller = ioloop.ZMQPoller()
        poller.register(req, ioloop.IOLoop.READ)
        poller.register(rep, ioloop.IOLoop.READ)
        events = dict(poller.poll(0))
        self.assertEquals(events.get(rep), None)
        self.assertEquals(events.get(req), None)

        poller.register(req, ioloop.IOLoop.WRITE)
        poller.register(rep, ioloop.IOLoop.WRITE)
        events = dict(poller.poll(1))
        self.assertEquals(events.get(req), ioloop.IOLoop.WRITE)
        self.assertEquals(events.get(rep), None)

        poller.register(rep, ioloop.IOLoop.READ)
        req.send(b'hi')
        events = dict(poller.poll(1))
        self.assertEquals(events.get(rep), ioloop.IOLoop.READ)
        self.assertEquals(events.get(req), None)

